/// Redis Connector
//
use async_trait::async_trait;
use anyhow::{anyhow,Result};

use crate::defs::{StoreSettings,PoolHandle};

#[derive(Default)]
pub struct RedisPool {
	pub id: String,
  pub client: Option<redis::Client>,
	pub conn: Option<redis::aio::Connection>,
}

impl std::fmt::Debug for RedisPool {
  fn fmt(&self, fmt: &mut std::fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
		write!(fmt, "RedisPool client: {:?}", self.client)
  }
}

impl RedisPool {
	pub fn new(pool_settings: StoreSettings) -> Self {
		let url = pool_settings.url_keyval();
		let debug = envmnt::get_isize("DEBUG",0);
		if debug > 0 {
			println!("Open Redis {}: {}",pool_settings.id, &url);
		}
		match redis::Client::open(url) {
			Ok(cli) => Self {
				id: pool_settings.id,
				client: Some(cli.to_owned()),
				conn: None,
			},
			Err(e) => { 
				eprintln!("Error Redis new pool: {}",e);
				Self::default()
			},
		}
	}
	pub async fn connect_pool(self) -> Self {
		self.connect().await
	}
	pub async fn get_conn(self) -> Result<redis::aio::Connection> {
		if let Some(pool) = self.conn {
			Ok(pool)
		} else {
			let cli=self.connect_pool().await;
		  if let Some(pool) = cli.conn {
				Ok(pool)
			} else {
				Err(anyhow!("Redis pool not available"))
			}
		}
	}
}
#[async_trait]
impl PoolHandle for RedisPool {
	async fn connect(&self) -> Self { 
		let cli = self;
		let mut conn = None; 
		if let Some(client) = &cli.client {
			match	client.get_async_connection().await {
				Ok(cli_pool) => {
					let debug = envmnt::get_isize("DEBUG",0);
					if debug > 0 {
			    	println!("Got Redis connection");
					}
					conn = Some(cli_pool);
				},
				Err(e) => {
					eprintln!("Error Redis connection: {}",e);
				}
			};
		}
		Self {
			id: cli.id.to_owned(),
			client: cli.client.to_owned(),
			conn,
		}
	}	

}
