///  Sqlite Connection
//
use anyhow::{Result,anyhow};
use async_trait::async_trait;

use crate::defs::{StoreSettings,PoolHandle};

#[derive(Default)]
pub struct SqlitePool {
	pub id: String,
  // TODO Pool with Sqlite not working
  // pub client: Option<sqlx::Pool<sqlx::Sqlite>>,
  // 	pub conn: Option<sqlx::pool::PoolConnection<sqlx::Sqlite>>,
  pub client: Option<String>,
  pub conn: Option<sqlx::Pool<sqlx::Sqlite>>,
}

impl std::fmt::Debug for SqlitePool {
  fn fmt(&self, fmt: &mut std::fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
		write!(fmt, "SqlitePool client: {:?}", self.client)
  }
}

impl SqlitePool {
	pub fn new(pool_settings: StoreSettings) -> Self {
		let url = pool_settings.url_local();
		let debug = envmnt::get_isize("DEBUG",0);
		if debug > 0 {
			println!("Open Sqlite {}: {}",pool_settings.id, &url);
		}
	 	Self {
	 	 	id: pool_settings.id.to_owned(),
	 	 	client: Some(url),
	 	 	conn: None,
		 }
	}
	pub async fn connect_pool(self) -> Self {
		self.connect().await
	}
	pub async fn get_conn(self) -> Result<sqlx::Pool<sqlx::Sqlite>> {
		if let Some(pool) = self.conn {
			Ok(pool)
		} else {
			let cli=self.connect_pool().await;
		  if let Some(pool) = cli.conn {
				Ok(pool)
			} else {
				Err(anyhow!("Sqlite pool not available"))
			}
		}
	}
}
#[async_trait]
impl PoolHandle for SqlitePool {
	async fn connect(&self) -> Self {
	 	let cli = self;
		let mut conn = None; // DataPool::NoPool;
		if let Some(client) = &cli.client {
			// TODO Pool with Sqlite not working
			// 	 match client.acquire().await {
			match sqlx::sqlite::SqlitePool::connect(&client).await {
				Ok(pool) => {
					let debug = envmnt::get_isize("DEBUG",0);
					if debug > 0 {
						println!("Got Sqlite connection");
					}
					conn = Some(pool);
				},
				Err(e) => { 
					eprintln!("Error Sqlite new pool: {}",e);
				},
			};
		}
	 	Self {
	 	 	id: cli.id.to_owned(),
	 	 	client: cli.client.to_owned(),
	 	 	conn,
	 	}
	}	
}