/// Tikv Connection
//
// Copyright 2020, Jesús Pérez Lorenzo
//
use crate::models::Tikv;
use anyhow::Context as AnyContext;
use std::path::PathBuf;
use tikv_client::{Config, Key, KvPair, RawClient, Result, TransactionClient, ToOwnedRange, Value};
use crate::models::config::StoreKeyValue;

/// Tikv pool connection
pub async fn get_tikv_pool(tikv: &Tikv) -> anyhow::Result<RawClient> {
  let tikv_url = config.url("tikv");
  // Optionally encrypt the traffic.
  let config = if let (Some(ca), Some(cert), Some(key)) = (tikv.ca, tikv.cert, tikv.key) {
    Config::new(tikv.pd).with_security(ca, cert, key)
  } else {
    Config::new(tikv.pd)
  };
  // When we first create a client we receive a `Connect` structure which must be resolved before
  // the client is actually connected and usable.
  // let client = if tikv.trans {
  //   TransactionClient::new(config).await?
  //   //let mut txn = txn_client.begin().await?;
  // } else {
  let client = RawClient::new(config)
    .await
    .context(format!("get tikv new tikv pool {}", redis_url))?;
  //}
  Ok(client)
}
